@@ -194,7 +194,7 @@ static void fix_msdos_allcaps(LHAFileHeader *header)
 
 	if (header->path != NULL) {
 		for (i = 0; header->path[i] != '\0'; ++i) {
-			if (islower((unsigned) header->path[i])) {
+			if (islower((int)(unsigned char) header->path[i])) {
 				is_allcaps = 0;
 				break;
 			}
@@ -203,7 +203,7 @@ static void fix_msdos_allcaps(LHAFileHeader *header)
 
 	if (is_allcaps && header->filename != NULL) {
 		for (i = 0; header->filename[i] != '\0'; ++i) {
-			if (islower((unsigned) header->filename[i])) {
+			if (islower((int)(unsigned char) header->filename[i])) {
 				is_allcaps = 0;
 				break;
 			}
@@ -216,13 +216,13 @@ static void fix_msdos_allcaps(LHAFileHeader *header)
 		if (header->path != NULL) {
 			for (i = 0; header->path[i] != '\0'; ++i) {
 				header->path[i]
-				    = tolower((unsigned) header->path[i]);
+				    = tolower((int)(unsigned char) header->path[i]);
 			}
 		}
 		if (header->filename != NULL) {
 			for (i = 0; header->filename[i] != '\0'; ++i) {
 				header->filename[i]
-				    = tolower((unsigned) header->filename[i]);
+				    = tolower((int)(unsigned char) header->filename[i]);
 			}
 		}
 	}
@@ -1011,6 +1011,7 @@ LHAFileHeader *lha_file_header_read(LHAInputStream *stream)
 	if (header->os_type == LHA_OS_TYPE_UNKNOWN
 	 || header->os_type == LHA_OS_TYPE_MSDOS
 	 || header->os_type == LHA_OS_TYPE_ATARI
+	 || header->os_type == LHA_OS_TYPE_LHARK
 	 || header->os_type == LHA_OS_TYPE_OS2) {
 		fix_msdos_allcaps(header);
 	}
@@ -1044,6 +1045,15 @@ LHAFileHeader *lha_file_header_read(LHAInputStream *stream)
 		goto fail;
 	}
 
+	// The DOS LHARK tool has its own -lh7- format that is incompatible
+	// with the -lh7- that everyone else uses. As a workaround, we detect
+	// and rename the compression method to -lk7- so as to be able to
+	// distinguish between the two formats.
+	if (header->header_level == 1 && header->os_type == LHA_OS_TYPE_LHARK
+	 && !strncmp(header->compress_method, "-lh7-", 5)) {
+		header->compress_method[2] = 'k';
+	}
+
 	return header;
 fail:
 	lha_file_header_free(header);
@@ -1078,4 +1088,3 @@ void lha_file_header_add_ref(LHAFileHeader *header)
 {
 	++header->_refcount;
 }
-
